module.exports = {
    masterInit: (app) => {
        return (routePrefix, pushEdit, editCheck, router, model, permission) => {
            return {
                string: async function (req, res, name, match) {
                    if (!req.params.id) {return;}
                    const document = await model.findOne({id: req.params.id.toLowerCase()});
                    if (!editCheck(document, req, res)) {return;}
                    const camelName = app.util.bodyCase(name);
                    if (!req.body[camelName]) {return res.status(400).send(`You did not provide a new ${camelName} in your body.`);}
                    if (!match(req.body[camelName])) {return res.status(400).send(`Your new ${camelName} is too long or contains illegal characters.`);}
                    try {
                        document[camelName] = req.body[camelName].trim();
                        pushEdit(`Updated ${camelName}`, req, document);
                        return document.save()
                            .then(() => res.send(`${camelName.charAt(0).toUpperCase()}${camelName.slice(1)} updated.`))
                            .catch(() => res.status(500).send(`There was an error trying to update your ${camelName}. Please try again.`));
                    } catch {return res.status(500).send(`There was an error trying to update your ${camelName}. Please try again.`);}
                },

                stringWrap: function (name, match, forcePermission) {
                    const route = router.route(`${routePrefix}/${name}`);
                    route.patch(app.auth.token, app.auth.permsPass(forcePermission || permission), async (req, res) => {
                        await this.string(req, res, name, match).catch(() => res.status(500).send(`There was an error trying to update your ${name}. Please try again.`));
                    });
                    return route
                },

                list: function (name, bodyName, match, forcePermission, deepRoute) {
                    const camelName = app.util.bodyCase(name);
                    router.use(`${routePrefix}/${name}`, app.auth.tokenPass, app.auth.permsPass(forcePermission || permission), async (req, res, next) => {
                        if (!req.params.id) {return await res.status(400).send("Missing ID!");}
                        const document = await model.findOne({id: req.params.id.toLowerCase()});
                        if (!editCheck(document, req, res)) {return;}
                        req.document = document;
                        req.listData = deepRoute ? document[deepRoute][camelName] : document[camelName];
                        next();
                    }, app.util.list(router, `${routePrefix}/${name}`, true, bodyName, async (req, res, list, item) => {
                        if (item && !match(item)) {res.status(400).send(`The ${camelName} you provided is invalid.`); return 0;}
                        if (item) {list[list.length - 1] = item;}
                        if (deepRoute) {req.document[deepRoute][camelName] = list} else {req.document[camelName] = list;}
                        req.document.markModified(deepRoute ? `${deepRoute}.${camelName}` : camelName);
                        await req.document.save();
                    }, (req, res, next) => {
                        if (!req.authenticatedUser) {return res.status(401).send("You must be authenticated before you do that!");}
                        if (req.unauthorized) {return res.status(401).send("You are not authorized to edit that!");}
                        return next();
                    }));
                }
            }
        }
    }
}